<?
//!-----------------------------------------------------------------
// @function        captureGlobalInfo
// @desc            Captura as informaes atuais da pgina para
//                  montar um snapshot da situao do sistema no
//                  momento em que ocorre um erro
// @returns         info      data, hora, get, post e sesso atual
//!-----------------------------------------------------------------
function captureGlobalInfo()
{
     global $PS_USER_CFG,$PS_LANGUAGE;

     $info = "";
     $getPars = 0;
     $postPars = 0;

     // data e hora
     $info .= date("d/m/Y - H:i:s")."|";

     // parmetros get
     $getVarsNaming = (isset($PS_LANGUAGE['GET_PARAMETER_NAMING'])) ? $PS_LANGUAGE['GET_PARAMETER_NAMING'] : "Parmetro Get";
     while (list($key,$value)=each($_GET)) {
          $info .= $getVarsNaming." ".(++$getPars).": ".$key."=>".$value."|";
     }

     // parmetros post
     $postVarsNaming = (isset($PS_LANGUAGE['POST_PARAMETER_NAMING'])) ? $PS_LANGUAGE['POST_PARAMETER_NAMING'] : "Parmetro Post";
     while (list($key,$value)=each($_POST)) {
          $info .= $postVarsNaming." ".(++$postPars).": ".$key."=>".$value."|";
     }

     // usurio de sesso
     $sessionUserNaming = (isset($PS_LANGUAGE['SESSION_USER_NAMING'])) ? $PS_LANGUAGE['SESSION_USER_NAMING'] : "Usurio de Sesso";
     if (isset($PS_USER_CFG['SESSION_OBJECT_NAME'])) {
          if (isset($_SESSION[$PS_USER_CFG['SESSION_OBJECT_NAME']])) {
               $SESSION_OBJECT = $_SESSION[$PS_USER_CFG['SESSION_OBJECT_NAME']];
               if (method_exists($SESSION_OBJECT,"getBasicInfo")) {
                    $info .= $sessionUserNaming." : ".$SESSION_OBJECT->getBasicInfo()."|";
               }
          }
     }
     return $info;
}

//------------------------------------------------------------------
// @function        phpScribeErrorHandler
// @desc            Substitui o tratador de erros tradicional do PHP,
//                  buscando informaes momentneas do sistema para
//                  posterior anlise das condies do erro
// @param           errno          nmero do erro
// @param           errmsg         mensagem de erro
// @param           filename       arquivo onde o erro ocorreu
// @param           linenum        nmero da linha no arquivo
// @param           vars           tabela de smbolos no momento do erro
//------------------------------------------------------------------
function phpScribeErrorHandler ($errno, $errmsg, $filename, $linenum, $vars)
{
     global $PS_USER_CFG,$PS_LANGUAGE;

     // busca da configurao de linguagem o array associativo com os nomes dos tipos de erros
     $errorTypes = $PS_LANGUAGE['ERR_TYPES'];

     // define mensagens de erro que devem ser ignoradas
     $ignoreErrors = array("UNDEFINED INDEX","USE OF UNDEFINED CONSTANT");

     // verifica se o erro capturado no corresponde
     // a algum tipo de erro que deve ser ignorado
     $triggerError = TRUE;
     for ($i=0; $i<count($ignoreErrors); $i++) {
          if (ereg($ignoreErrors[$i],strtoupper($errmsg))) {
               $triggerError = FALSE;
          }
     }
     if ($triggerError) {
          // cria um vetor com os tipos de erros do usurio
          $userErrors = array(E_USER_ERROR, E_USER_WARNING, E_USER_NOTICE);

          // armazena os dados capturados pelo tratador de erros
          // nmero do erro, tipo do erro, mensagem de erro, arquivo e linha do cdigo
          $err = $errno."|";
          $err .= $errorTypes[$errno]."|";
          $err .= $errmsg."|";
          $err .= $filename."|";
          $err .= $linenum."|";

          // realiza var trace para erros de usurio
          if (in_array($errno, $userErrors)) {
               ob_start();
               var_dump($vars);
               $err .= ob_get_contents();
               ob_end_clean();
          }

          // imprime o erro para visualizao
          if ((isset($PS_USER_CFG['SHOW_ERRORS'])) && ($PS_USER_CFG['SHOW_ERRORS'] == TRUE)) {
               friendlyErrorMessage($err,$errorTypes[$errno]);
          }

          // salva no log de erros
          if ((isset($PS_USER_CFG['LOG_ERRORS'])) && ($PS_USER_CFG['LOG_ERRORS'] == TRUE)) {
               if ((isset($PS_USER_CFG['ERROR_LOG_FILE'])) && (is_writeable($PS_USER_CFG['ERROR_LOG_FILE']))) {
                    error_log(captureGlobalInfo().$err."<*>\n", 3, $PS_USER_CFG['ERROR_LOG_FILE']);
               } else {
                    error_log(captureGlobalInfo().$err."<*>\n", 0);
               }
          }

          // cancela a execuo do script se o erro for E_USER_ERROR
          if ($errno == E_USER_ERROR) {
               exit;
          }
     }
}

//------------------------------------------------------------------
// @function        friendlyErrorMessage
// @desc            Imprime as informaes sobre o erro capturado
// @param           err       dados do erro capturado
// @param           type      tipo do erro capturado
//------------------------------------------------------------------
function friendlyErrorMessage($err,$type) {
     $errData = explode("|",$err);
     echo "<table cellspacing='0' border='1' bordercolor='#000000'>\n".
          "<tr bgcolor='#e2e2e2'><td align='left'><font color='#ff0000'>&nbsp;<b>".strtoupper($type)." :</b></font></td></tr>\n".
          "<tr><td align='left'>".
          "<B>".$errData[2]."</B><BR>".
          "FILE: ".$errData[3].", LINE: ".$errData[4].
          "</td></tr>\n".
          "</table>\n";
}

//------------------------------------------------------------------
// @function        dbErrorHandler
// @desc            Tratador de erros para a bilioteca ADODB
// @param           dbms           nome do banco de dados utilizado
// @param           function       funo onde ocorreu o erro
// @param           errno          nmero do erro de banco
// @param           errmsg         mensagem de erro do banco
// @param           p1             parmetro 1 (mixed)
// @param           p2             parmetro 2 (mixed)
//------------------------------------------------------------------
function dbErrorHandler($dbms, $function, $errno, $errmsg, $p1=false, $p2=false)
{
     global $PS_USER_CFG,$PS_LANGUAGE;

     // monta o informativo do erro com o nome do banco utilizado,
     // nmero e mensagem do erro e funo executada
     $err = $dbms."|";
     $err .= $errno."|";
     $err .= $errmsg."|";
     switch($function) {
          case 'EXECUTE':$sql = $p1;
                         $inputparams = $p2;
                         $err .= "$function(\"$sql\")|";
                         break;
          case 'PCONNECT':
          case 'CONNECT':$host = $p1;
                         $database = $p2;
                         $err = "$function($host, '****', '****', $database)|";
                         break;
          default:       $err = "$function($p1, $p2)|";
                         break;
     }

     // imprime o erro para visualizao
     if ((isset($PS_USER_CFG['SHOW_ERRORS'])) && ($PS_USER_CFG['SHOW_ERRORS'] == TRUE)) {
          friendlyDbErrorMessage($err,$PS_LANGUAGE['ERR_DATABASE']);
     }

     // salva no log de erros
     if ((isset($PS_USER_CFG['LOG_ERRORS'])) && ($PS_USER_CFG['LOG_ERRORS'] == TRUE)) {
          if ((isset($PS_USER_CFG['DB_ERROR_LOG_FILE'])) && (is_writeable($PS_USER_CFG['DB_ERROR_LOG_FILE']))) {
               error_log(captureGlobalInfo().$err."<*>\n", 3, $PS_USER_CFG['DB_ERROR_LOG_FILE']);
          } else {
               error_log(captureGlobalInfo().$err."<*>\n", 0);
          }
     }

}

//------------------------------------------------------------------
// @function        friendlyDbErrorMessage
// @desc            Imprime as informaes sobre o erro de banco
// @param           err       dados do erro capturado
// @param           type      tipo do erro capturado
//------------------------------------------------------------------
function friendlyDbErrorMessage($err,$type) {
     $errData = explode("|",$err);
     echo "<table cellspacing='0' border='1' bordercolor='#000000'>\n".
          "<tr bgcolor='#e2e2e2'><td align='left'><font color='#ff0000'>&nbsp;<b>".strtoupper($type)." :</b></font></td></tr>\n".
          "<tr><td align='left'>".
          "<B>".$errData[2]."</B><BR>".
          "DBMS: ".$errData[0].", ERRNO: ".$errData[1]."<BR>".
          "FUNCTION: ".$errData[3].
          "</td></tr>\n".
          "</table>\n";
}
?>